import { EventEmitter } from 'events';
import { v4 as uuid } from 'uuid';
import { Client, Collection } from '.';

export type QueueID = string;

export interface QueueEntry {
  readonly id?: QueueID,
  readonly enteredDate?: Date,
  readonly submittedBy: string,
  func: Function,
}

export default class Queue extends EventEmitter {
  public queueName: string;

  public client: Client;

  protected set: Collection<QueueEntry>;

  constructor(client: Client, queueName: string) {
    super();
    this.client = client;
    this.queueName = queueName;
    this.set = new Collection();
  }

  public postListeners() {
    this.on('newEntry', (data: QueueEntry) => {
      this.client.util.signale.info(`New Entry in Queue ${this.queueName}\nEntered Date: ${data.enteredDate} | Submitted by: ${data.submittedBy}`);
    });
    this.on('processing', (data: QueueEntry) => {
      this.client.util.signale.pending(`Processing Entry in Queue ${this.queueName}\nEntered Date: ${data.enteredDate} | Submitted by: ${data.submittedBy}`);
    });
    this.on('done', (data: QueueEntry) => {
      this.client.util.signale.success(`Finished Entry in Queue ${this.queueName}\nEntered Date: ${data.enteredDate} | Submitted by: ${data.submittedBy}`);
    });
  }

  public add(entry: QueueEntry) {
    const data = this.set.add(uuid(), { enteredDate: new Date(), submittedBy: entry.submittedBy, func: entry.func });
    this.emit('newEntry', data);
    this.do(data);
    return data;
  }

  protected async do(entry: QueueEntry) {
    try {
      this.emit('processing', entry);
      await entry.func(this.client);
    } finally {
      this.set.delete(entry.id);
      this.emit('done', entry);
    }
  }
}
